class CollectionResolver:
    def preorder_resolve(self, node, root_node, result, trace, root_resolver):
        pass

    def postorder_resolve(self, node, root_node, result, trace, root_resolver):
        pass